@@ -105,6 +105,10 @@ public class ClassWriterTest extends AsmTest {
                 "nestHostClassIndex",
                 "numberOfNestMemberClasses",
                 "nestMemberClasses",
+                "numberOfPermittedSubclasses",
+                "permittedSubclasses",
+                "firstRecordComponent",
+                "lastRecordComponent",
                 "firstAttribute",
                 "compute"));
     // IMPORTANT: if this fails, update the string list AND update the logic that resets the
