@@ -40,6 +40,7 @@ import java.util.Arrays;
 import java.util.List;
 import java.util.Set;
 import java.util.stream.Collectors;
+import java.util.stream.Stream;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.function.Executable;
 
@@ -215,7 +216,9 @@ public class ConstantsTest {
   }
 
   private static List<Field> getConstants(final ConstantType constantType) {
-    return Arrays.asList(Constants.class.getFields()).stream()
+    return Stream.concat(
+            Arrays.asList(Opcodes.class.getFields()).stream(),
+            Arrays.asList(Constants.class.getFields()).stream())
         .filter(field -> getType(field).equals(constantType))
         .collect(Collectors.toList());
   }
@@ -226,7 +229,9 @@ public class ConstantsTest {
       case "ASM5":
       case "ASM6":
       case "ASM7":
-      case "ASM8_EXPERIMENTAL":
+      case "ASM8":
+      case "ASM9":
+      case "ASM10_EXPERIMENTAL":
         return ConstantType.ASM_VERSION;
       case "V_PREVIEW":
       case "V1_1":
@@ -244,6 +249,7 @@ public class ConstantsTest {
       case "V13":
       case "V14":
       case "V15":
+      case "V16":
         return ConstantType.CLASS_VERSION;
       case "ACC_PUBLIC":
       case "ACC_PRIVATE":
@@ -268,9 +274,10 @@ public class ConstantsTest {
       case "ACC_ENUM":
       case "ACC_MANDATED":
       case "ACC_MODULE":
-        return ConstantType.ACCESS_FLAG;
-      case "ACC_DEPRECATED":
       case "ACC_SEALED":
+        return ConstantType.ACCESS_FLAG;
+      case "ACC_RECORD":
+      case "ACC_DEPRECATED":
       case "ACC_CONSTRUCTOR":
         return ConstantType.ASM_ACCESS_FLAG;
       case "T_BOOLEAN":
